# encoding: utf-8
from __future__ import annotations

import json
import logging
from json import JSONDecodeError
from typing import Dict

import requests
from requests import Response
from requests.auth import HTTPBasicAuth

from seller_active.__info__ import __package_name__, __version__

from .helpers import params_override

logger = logging.getLogger(__name__)


class BaseClient:
    SCHEME = "https://"
    HOST = "rest.selleractive.com"
    API_VERSION = "api"
    USER_AGENT = f"{__package_name__}-{__version__}"

    def __init__(self, seller_id: str, api_key: str):
        self.seller_id = seller_id
        self.api_key = api_key

        self.session = requests.Session()
        self.session.headers = self.headers
        self.session.auth = self.basic_auth

    @property
    def basic_auth(self) -> HTTPBasicAuth:
        return HTTPBasicAuth(self.seller_id, self.api_key)

    @property
    def headers(self):
        return {
            "user-agent": self.USER_AGENT,
            "host": self.HOST,
            "accept": "application/json",
            "content-type": "application/json",
        }

    @property
    def endpoint(self):
        return self.endpoint

    def _request(
        self,
        method: str,
        data=None,
        params: dict | None = None,
        headers: dict | None = None,
        timeout: float | int | None = None,
    ) -> Dict:
        if params is None:
            params = {}
        else:
            params = params_override(params)
        if data is None:
            data = {}

        resp = self.session.request(
            method,
            f"{self.SCHEME}{self.HOST}/{self.API_VERSION}/{self.endpoint}",
            params=params,
            data=json.dumps(data) if data and method in ("POST", "PUT", "PATCH") else None,
            headers=headers,
            timeout=timeout,
        )
        return self.request_to_api_response(resp)

    @staticmethod
    def request_to_api_response(response: Response) -> Dict:
        try:
            js = response.json() or {}
        except JSONDecodeError:
            if response.request.method in ("DELETE", "PUT") and 200 <= response.status_code < 300:
                js = {}
            else:
                js = {"status_code": response.status_code}

        if isinstance(js, list):
            js = dict(data=js)

        error = js.get("Message", None)

        rate_limits = {
            "rate_limit": response.headers.get("X-RateLimit-Limit", None),
            "rate_limit_remaining": response.headers.get("X-RateLimit-Remaining", None),
            "rate_limit_reset": response.headers.get("X-RateLimit-Reset", None),
        }

        return {"data": js, "error": error, "headers": dict(response.headers), "rate_limits": rate_limits}



# encoding: utf-8
from datetime import datetime
from typing import Dict, Optional

from seller_active.base import BaseClient


class Locations(BaseClient):
    endpoint = "Location"

    def get_locations(
        self,
        sku: Optional[str] = None,
        location_name: Optional[str] = None,
        created_after: Optional[datetime] = None,
        created_before: Optional[datetime] = None,
        updated_after: Optional[datetime] = None,
        updated_before: Optional[datetime] = None,
        page: int = 1,
        page_size: int = 100,
    ):
        """
        Retrieve list of inventory locations using provided filter parameters Requests for pages > 1,000 will be
        rejected. Please use the parameters (e.g. updated_after) to reduce the data returned.
        """
        params = {
            k: v
            for k, v in {
                "page": page,
                "page_size": page_size,
                "sku": sku,
                "location_name": location_name,
                "created_after": created_after,
                "created_before": created_before,
                "updated_after": updated_after,
                "updated_before": updated_before,
            }.items()
            if v is not None
        }

        return self._request("GET", params=params)

    def update_location(self, data: Dict):
        """
        Modify an existing inventory location, must specify the location by both
        required attributes 'SKU' and 'LocationName'.
        data = {
          "SKU": "string",
          "LocationName": "string",
          "Quantity": 0,
          "Priority": 0,
          "Type": "None",
          "Integration": "None"
        }
        """
        if "SKU" not in data.keys() or "LocationName" not in data.keys():
            raise ValueError("'SKU' or 'LocationName' attribute(s) was not found in `data`.")
        return self._request("PUT", data=data)

    def delete_location(self, sku: str, location_name: str):
        """
        Delete an existing inventory location, must specify the location by both
        required attributes 'SKU' and 'LocationName'.
        """
        params = {
            "sku": sku,
            "location_name": location_name,
        }
        return self._request("DELETE", params=params)

    def create_location(self, data: Dict):
        """
        Adds a new location to an existing inventory item, attribute 'SKU' is required to specify the item and
        quantity of location must be greater than or equal to 0. Location names should be unique.
        data = {
          "SKU": "string",
          "LocationName": "string",
          "Quantity": 0,
          "Priority": 0,
          "Type": "None",
          "Integration": "None"
        }
        """
        required_keys = {"SKU", "LocationName", "Quantity", "Priority"}
        if not required_keys.issubset(data.keys()):
            missing_keys = [k for k in required_keys if k not in data.keys()]
            raise ValueError(f"{missing_keys} attribute(s) was not found in `data`.")

        return self._request("POST", data=data)



# encoding: utf-8
from datetime import datetime
from typing import Dict, Optional

from seller_active.base import BaseClient


class Orders(BaseClient):
    endpoint = "Order"

    def get_orders(
        self,
        site_order_id: Optional[str] = None,
        seller_order_id: Optional[str] = None,
        sa_order_id: Optional[str] = None,
        site_item_id: Optional[str] = None,
        sku: Optional[str] = None,
        site: Optional[str] = None,
        order_status: Optional[str] = None,
        upc: Optional[str] = None,
        vendor: Optional[str] = None,
        shipped_after: Optional[datetime] = None,
        shipped_before: Optional[datetime] = None,
        ordered_after: Optional[datetime] = None,
        ordered_before: Optional[datetime] = None,
        created_after: Optional[datetime] = None,
        created_before: Optional[datetime] = None,
        updated_after: Optional[datetime] = None,
        updated_before: Optional[datetime] = None,
        page: int = 1,
        page_size: int = 100,
    ):
        """
        Retrieve list of orders using provided filter parameters. Requests for pages > 1,000 will be
        rejected. Please use the parameters (e.g. updated_after) to reduce the data returned.
        """
        params = {
            k: v
            for k, v in {
                "page": page,
                "page_size": page_size,
                "site_order_id": site_order_id,
                "seller_order_id": seller_order_id,
                "sa_order_id": sa_order_id,
                "site_item_id": site_item_id,
                "sku": sku,
                "site": site,
                "order_status": order_status,
                "upc": upc,
                "vendor": vendor,
                "shipped_after": shipped_after,
                "shipped_before": shipped_before,
                "ordered_after": ordered_after,
                "ordered_before": ordered_before,
                "created_after": created_after,
                "created_before": created_before,
                "updated_after": updated_after,
                "updated_before": updated_before,
            }.items()
            if v is not None
        }

        return self._request("GET", params=params)

    def update_order(self, data: Dict):
        """
        Modifies an existing order, orders are specified using the combination of required attributes 'Site' and
        'SiteOrderID'. Specific order items are related back using 'SiteItemID'. Ensure orders exist before
        attempting to modify. To quickly fulfill an order (Post tracking information and mark as shipped) use the
        additional fields 'Status', 'OrderTracking', 'OrderService', and 'OrderCarrier'. This will post the shipping
        information to all order details related to that order.
        """
        if "Site" not in data.keys() or "SiteOrderID" not in data.keys():
            raise ValueError("'Site' or 'SiteOrderID' attribute(s) was not found in `data`.")
        return self._request("PUT", data=data)

    def delete_order(self, site_order_id: str, site: str, site_item_id: Optional[str] = None):
        """
        Deletes an existing order specified by required attributes 'SiteOrderID' and 'Site'. Attribute 'SiteItemID'
        is an optional field, if specified the action will delete the single order item, otherwise if not specified
        the entirety of the order is deleted.
        """
        params = {
            "site_order_id": site_order_id,
            "site": site,
            "site_item_id": site_item_id,
        }
        return self._request("DELETE", params=params)

    def create_order(self, data: Dict):
        """
        Creates a new order, Site Order ID must be unique per marketplace. Required fields include SiteOrderID, Site,
        OrderStatus, and SiteItemID (On the order detail level). Requires at least one order detail specified.
        """
        required_keys = {"SiteOrderID", "Site", "OrderStatus", "SiteItemID", "OrderDetails"}
        if not required_keys.issubset(data.keys()):
            missing_keys = [k for k in required_keys if k not in data.keys()]
            raise ValueError(f"{missing_keys} attribute(s) was not found in `data`.")

        return self._request("POST", data=data)



# encoding: utf-8
from datetime import datetime
from typing import Dict, Optional

from seller_active.base import BaseClient


class Bundles(BaseClient):
    endpoint = "Bundle"

    def get_bundles(
        self,
        bundle_sku: Optional[str] = None,
        individual_sku: Optional[str] = None,
        created_after: Optional[datetime] = None,
        created_before: Optional[datetime] = None,
        updated_after: Optional[datetime] = None,
        updated_before: Optional[datetime] = None,
        page: int = 1,
        page_size: int = 100,
    ):
        """
        Retrieve list of bundles using provided filter parameters. Requests for pages > 1,000 will be
        rejected. Please use the parameters (e.g. updated_after) to reduce the data returned.
        """
        params = {
            k: v
            for k, v in {
                "page": page,
                "page_size": page_size,
                "bundle_sku": bundle_sku,
                "individual_sku": individual_sku,
                "created_after": created_after,
                "created_before": created_before,
                "updated_after": updated_after,
                "updated_before": updated_before,
            }.items()
            if v is not None
        }

        return self._request("GET", params=params)

    def update_bundle(self, data: Dict):
        """
        Modifies an existing inventory bundle, all SKUs and relationships must exist prior. Quantity and price values
        are adjusted through the 'Inventory' POST API operations, 'Bundle' POST API operations modify the
        relationship amounts of individual SKUs in each bundle.
        data = {
          "BundleSKU": "string",
          "BundledItems": [
            {
              "SKU": "string",
              "AmountPerBundle": 0
            }
          ]
        }
        """
        if "BundleSKU" not in data.keys() or "BundledItems" not in data.keys():
            raise ValueError("'BundleSKU' or 'BundledItems' attribute was not found in `data`.")
        return self._request("PUT", data=data)

    def delete_inventory(self, bundle_sku: str, individual_sku: Optional[str] = None):
        """
        Deletes an existing bundle relationship. BundleSKU attribute is required to specify the bundle, individual
        SKU is an optional attribute. Specifying the individual SKU will remove that single item from the bundle
        relationship, specifying only the bundle SKU will remove all items from the bundle relationship.

        """
        params = {
            k: v
            for k, v in {
                "bundle_sku": bundle_sku,
                "individual_sku": individual_sku,
            }.items()
            if v is not None
        }
        return self._request("DELETE", params=params)

    def create_bundle(self, data: Dict):
        """
        Creates a new inventory bundle relationship, all SKUs specified must exist in inventory table prior.
        Limitations still apply, SKUs that have warehouse locations or are FBA items cannot be bundled. A bundle SKU
        cannot be contained in another bundle as an individual SKU, and an individual SKU cannot be specified as a
        bundle SKU.
        data = {
          "BundleSKU": "string",
          "BundledItems": [
            {
              "SKU": "string",
              "AmountPerBundle": 0
            }
          ]
        }
        """
        required_keys = {"BundleSKU", "BundledItems"}
        if not required_keys.issubset(data.keys()):
            missing_keys = [k for k in required_keys if k not in data.keys()]
            raise ValueError(f"{missing_keys} attribute(s) was not found in `data`.")

        return self._request("POST", data=data)



# encoding: utf-8
from .bundles.bundles import Bundles
from .inventory.inventory import Inventory
from .locations.locations import Locations
from .orders.orders import Orders
from .rate_limit_status.rate_limit_status import RateLimitStatus

__all__ = ["RateLimitStatus", "Inventory", "Bundles", "Locations", "Orders"]



# encoding: utf-8
from .__version__ import __version__

__author__ = "Yan Berezkin"
__maintainer__ = __author__
__package_name__ = "python_seller_active"
__email__ = "yan.berezkin@gmail.com"
__license__ = "MIT"
__version__ = __version__
__status__ = "Development"  # stable
__repo_url__ = "https://github.com/yberezkin/python_seller_active"
__bug_tracker__ = "https://github.com/yberezkin/python_seller_active/issues"
__credits__ = [""]



# encoding: utf-8
PARAMS_MAPPING = {
    "sku": "req.SKU",
    "individual_sku": "req.individualSKU",
    "bundle_sku": "req.bundleSKU",
    "title": "req.title",
    "site": "req.site",
    "location_name": "req.locationName",
    "vendor": "req.vendor",
    "site_order_id": "req.siteOrderID",
    "seller_order_id": "req.sellerOrderID",
    "sa_order_id": "req.orderID",
    "site_item_id": "req.siteItemID",
    "order_status": "req.orderStatus",
    "upc": "req.UPC",
    "page": "req.page",
    "page_size": "req.size",
    "created_after": "req.dateCreatedFrom",
    "created_before": "req.dateCreatedTo",
    "updated_after": "req.dateUpdatedFrom",
    "updated_before": "req.dateUpdatedTo",
    "shipped_after": "req.dateShippedFrom",
    "shipped_before": "req.dateShippedTo",
    "ordered_after": "req.dateOrderedFrom",
    "ordered_before": "req.dateOrderedTo",
}


def params_override(params):
    result = {PARAMS_MAPPING[k]: v for k, v in params.items()}
    return result



# encoding: utf-8
from datetime import datetime
from typing import Dict, Optional

from seller_active.base import BaseClient


class Inventory(BaseClient):
    endpoint = "Inventory"

    def get_inventory(
        self,
        sku: Optional[str] = None,
        title: Optional[str] = None,
        site: Optional[str] = None,
        vendor: Optional[str] = None,
        created_after: Optional[datetime] = None,
        created_before: Optional[datetime] = None,
        updated_after: Optional[datetime] = None,
        updated_before: Optional[datetime] = None,
        page: int = 1,
        page_size: int = 100,
    ):
        """
        Retrieve list of inventory items using provided filter parameters Note: Requests for pages > 1,000 will be
        rejected. Please use the parameters (e.g. updated_after) to reduce the data returned.
        """
        params = {
            k: v
            for k, v in {
                "page": page,
                "page_size": page_size,
                "sku": sku,
                "site": site,
                "title": title,
                "vendor": vendor,
                "created_after": created_after,
                "created_before": created_before,
                "updated_after": updated_after,
                "updated_before": updated_before,
            }.items()
            if v is not None
        }

        return self._request("GET", params=params)

    def update_inventory(self, data: Dict):
        """
        Modifies an existing inventory item, inventory is identified using required attribute 'SKU'.
        """
        if "SKU" not in data.keys():
            raise ValueError(
                "'SKU' attribute was not found in `data`."
                "Inventory Product is identified using required attribute 'SKU'"
            )
        return self._request("PUT", data=data)

    def delete_inventory(self, sku: str, site: Optional[str] = None):
        """
        Deletes an existing inventory item, or if attribute 'site' is specified deletes an existing listing under the
        specified marketplace. Only specifying SKU will delete all listings and the item.
        """
        params = {
            k: v
            for k, v in {
                "sku": sku,
                "site": site,
            }.items()
            if v is not None
        }
        return self._request("DELETE", params=params)

    def create_inventory(self, data: Dict):
        """
        Creates a new inventory item, SKU must be unique for every inventory item (Case insensitive), as well as
        at least one marketplace specified for the item to be listed under.

        Required fields include SKU, Condition, Title, ProductID, ProductTypy, Price, and Quantity
        """
        required_keys = {"SKU", "Condition", "Title", "ProductID", "ProductType", "Price", "Quantity"}
        if not required_keys.issubset(data.keys()):
            missing_keys = [k for k in required_keys if k not in data.keys()]
            raise ValueError(
                f"{missing_keys} attribute(s) was not found in `data`."
                f"Required fields include SKU, Condition, Title, ProductID, ProductTypy, Price, "
                f"and Quantity"
            )

        return self._request("POST", data=data)



# encoding: utf-8
from seller_active.base import BaseClient


class RateLimitStatus(BaseClient):
    endpoint = "RateLimitStatus"

    def get_rate_limit_status(self):
        return self._request("GET")



import setuptools

from seller_active import __info__

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name=__info__.__package_name__,
    version=__info__.__version__,
    author=__info__.__author__,
    author_email=__info__.__email__,
    description="Wrapper around Repricer.com API (aka XSellco)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=__info__.__repo_url__,
    project_urls={"Bug Tracker": __info__.__bug_tracker__},
    license=__info__.__license__,
    install_requires=["requests"],
    packages=[
        "seller_active",
        "seller_active.base",
        "seller_active.api",
        "seller_active.api.bundles",
        "seller_active.api.inventory",
        "seller_active.api.locations",
        "seller_active.api.orders",
        "seller_active.api.rate_limit_status",
    ],
)
